classdef BorderLine < matlab.graphics.chartcontainer.ChartContainer
    % Public property block     - UI
    % Private property block    - (Access = private / Transient / NonCopyable)
    % setup method              - sets the initial state of the chart.
    %                             It executes once when MATLAB® constructs
    %                             the object. Define this method in
    %                             a protected block so that only your class
    %                             can execute it.
    % update method             - This method updates the underlying objects
    %                             in your chart. It executes under
    %                             the following conditions:
    %                             - During the next drawnow execution, after
    %                             the user changes one or more property values
    %                             - When an aspect of the user's graphics
    %                             environment changes (such as the figure size)
    %                             - Define this method in the same protected
    %                             block as the setup method.
    % A constructor is inherited from the ChartContainer base class.
    % You can define a custom constructor method.
    % Convenience functions     - noncomputed: title,...
    %                           - computed: xlim,...
    
    properties
        XData (1,:) double = NaN
        YData (1,:) double = NaN
        BorderHigh (1,1) double = NaN
        MarkerSymbol = 'none'
        GraphUpColor = [1 0 0]
        GraphDownColor = [0 0 1]
        BorderColor = [0 1 0]
        TitleText = ''
        BorderWidth = 1
        GraphWidth = 0.5
    end
    
    properties(Access = private)
        BorderLineSelected = false
    end
    
    properties(Access = private,Transient,NonCopyable)
        BorderLineObject matlab.graphics.primitive.Line
        GraphUpLineObject matlab.graphics.chart.primitive.Line
        GraphDownLineObject matlab.graphics.chart.primitive.Line
    end
    
    properties (Dependent)
        XLimits (1,2) double
        XLimitsMode {mustBeMember(XLimitsMode,{'auto','manual'})} 
    end

    
    methods(Access = protected)
        function setup(obj)
            % Get the axes
            ax = getAxes(obj);
            
            % Create Patch and Line objects
            obj.BorderLineObject = line(ax,NaN,NaN,...
                'ButtonDownFcn', {@BorderLinePushCallback,obj});
            hold(ax,'on')
            obj.GraphUpLineObject = plot(ax,NaN,NaN);
            obj.GraphDownLineObject = plot(ax,NaN,NaN);
            
            % Turn hold state off
            hold(ax,'off')
        end
        
        function update(obj)
            
            % Update XData and YData of line object
            obj.GraphUpLineObject.XData = obj.XData;
            yUp = obj.YData;
            yUp(obj.YData < obj.BorderHigh) = NaN;
            obj.GraphUpLineObject.YData = yUp;
            
            obj.GraphDownLineObject.XData = obj.XData;
            yDown = obj.YData;
            yDown(obj.YData > obj.BorderHigh) = NaN;
            obj.GraphDownLineObject.YData = yDown;           
            
            % Update patch XData and YData
            ax = getAxes(obj);
            x = xlim(ax);
            obj.BorderLineObject.XData = x;
            lev = obj.BorderHigh;
            obj.BorderLineObject.YData = [lev, lev];
            
            % Update line object colors
            obj.GraphUpLineObject.Color = obj.GraphUpColor;
            obj.GraphDownLineObject.Color = obj.GraphDownColor;
            obj.BorderLineObject.Color = obj.BorderColor;
            
            % Update markers
            obj.GraphUpLineObject.Marker = obj.MarkerSymbol;
            obj.GraphDownLineObject.Marker = obj.MarkerSymbol;
            
            % Update width
            obj.GraphUpLineObject.LineWidth = obj.GraphWidth;
            obj.GraphDownLineObject.LineWidth = obj.GraphWidth;
            obj.BorderLineObject.LineWidth = obj.BorderWidth;
            
            % Call convenience functions
            title(getAxes(obj),obj.TitleText);
        end
    end
    
    methods
        function title(obj,txt)
            obj.TitleText = txt;
        end
        function varargout = xlim(obj,varargin)
            ax = getAxes(obj);
            [varargout{1:nargout}] = xlim(ax,varargin{:});
        end
        
        function set.XLimits(obj,xlm)
            ax = getAxes(obj);
            ax.XLim = xlm;
        end
        function xlm = get.XLimits(obj)
            ax = getAxes(obj);
            xlm = ax.XLim;
        end
        function set.XLimitsMode(obj,xlmmode)
            ax = getAxes(obj);
            ax.XLimMode = xlmmode;
        end
        function xlm = get.XLimitsMode(obj)
            ax = getAxes(obj);
            xlm = ax.XLimMode;
        end
        
    end
end
    
function BorderLinePushCallback(src,event,obj)
    obj.BorderLineSelected = true;
    fig = obj.Parent;
    fig.WindowButtonMotionFcn = {@BorderLineMotionCallback,obj};
    fig.WindowButtonUpFcn = {@BorderLineReleaseCallback,obj};
end

function BorderLineReleaseCallback(src,event,obj)
    if obj.BorderLineSelected
        ax = getAxes(obj);
        obj.BorderHigh = ax.CurrentPoint(1,2);
        obj.BorderLineSelected = false;
        src.WindowButtonMotionFcn = '';
        src.WindowButtonUpFcn = '';
    end
end

function BorderLineMotionCallback(src,event,obj)
    if obj.BorderLineSelected
        ax = getAxes(obj);
        obj.BorderHigh = ax.CurrentPoint(1,2);
        drawnow
    end
end