classdef ConfidenceChart < matlab.graphics.chartcontainer.ChartContainer
    % Public property block     - UI
    % Private property block    - (Access = private / Transient / NonCopyable)
    % setup method              - sets the initial state of the chart.
    %                             It executes once when MATLAB® constructs
    %                             the object. Define this method in
    %                             a protected block so that only your class
    %                             can execute it.
    % update method             - This method updates the underlying objects
    %                             in your chart. It executes under
    %                             the following conditions:
    %                             - During the next drawnow execution, after
    %                             the user changes one or more property values
    %                             - When an aspect of the user's graphics
    %                             environment changes (such as the figure size)
    %                             - Define this method in the same protected
    %                             block as the setup method.
    % A constructor is inherited from the ChartContainer base class.
    % You can define a custom constructor method.
    % Convenience functions     - noncomputed: title,...
    %                           - computed: xlim,...
    
    properties
        XData (1,:) double = NaN
        YData (1,:) double = NaN
        ConfidenceMargin = 0.15
        MarkerSymbol = 'o'
        Color = [1 0 0]
        TitleText = ''
    end
    
    properties(Access = private,Transient,NonCopyable)
        LineObject matlab.graphics.chart.primitive.Line
        PatchObject matlab.graphics.primitive.Patch
    end
    
    properties (Dependent)
        XLimits (1,2) double
        XLimitsMode {mustBeMember(XLimitsMode,{'auto','manual'})}
    end

    
    methods(Access = protected)
        function setup(obj)
            % Get the axes
            ax = getAxes(obj);
            
            % Create Patch and Line objects
            obj.PatchObject = patch(ax,NaN,NaN,'r','FaceAlpha',0.2,...
                'EdgeColor','none');
            hold(ax,'on')
            obj.LineObject = plot(ax,NaN,NaN);
            
            % Turn hold state off
            hold(ax,'off')
        end
        
        function update(obj)
            
            % Update XData and YData of line object
            obj.LineObject.XData = obj.XData;
            obj.LineObject.YData = obj.YData;
            
            % Update patch XData and YData
            x = obj.XData;
            obj.PatchObject.XData = [x x(end:-1:1)];
            y = obj.YData;
            c = obj.ConfidenceMargin;
            obj.PatchObject.YData = [y+c y(end:-1:1)-c];
            
            % Update line object colors
            obj.LineObject.Color = obj.Color;
            obj.PatchObject.FaceColor = obj.Color;
            
            % Update markers
            obj.LineObject.Marker = obj.MarkerSymbol;
            
            % Call convenience functions
            title(getAxes(obj),obj.TitleText);
        end
    end
    
    methods
        function title(obj,txt)
            obj.TitleText = txt;
        end
        function varargout = xlim(obj,varargin)
            ax = getAxes(obj);
            [varargout{1:nargout}] = xlim(ax,varargin{:});
        end
        
        function set.XLimits(obj,xlm)
            ax = getAxes(obj);
            ax.XLim = xlm;
        end
        function xlm = get.XLimits(obj)
            ax = getAxes(obj);
            xlm = ax.XLim;
        end
        function set.XLimitsMode(obj,xlmmode)
            ax = getAxes(obj);
            ax.XLimMode = xlmmode;
        end
        function xlm = get.XLimitsMode(obj)
            ax = getAxes(obj);
            xlm = ax.XLimMode;
        end
    end
        
end